#!/bin/sh
#
# The "log_integrate.sh" script.
# -----------------------------------------------------------------------------
# Copyright (c) 1995,1996 Donated to the public domain
# 
# Original author and maintainer: Marc Chatel   (mchatel@dial.oleane.com)
# Last known maintainer:          Marc Chatel   (mchatel@dial.oleane.com)
# 
# This file was created as part of the S4 (Secure System Setup Script) kit.
# Permission is granted to any person or entity to do any of the following:
#       a) use this file alone or in some other software
#       b) modify this file or include parts of this file in other files
#       c) re-distribute this file AS IS or modified, for non-commercial
#          or commercial purposes, alone or as part of some software package
# 
# No warranties of any kind, express or implied, on the functionality and safety
# of the contents of this file. Use at your own risk!
# 
# If you do useful changes to this file (bug fixes, portability fixes,
# enhancements), you should TRY to contact the current maintainer, who may be
# maintaining a "latest greatest" version of the file. You do not HAVE TO,
# but you should TRY. Promote software reuse! It helps everybody, including you!
# -----------------------------------------------------------------------------
#
# The "log_integrate.sh" script is intended to be run from CRON, typically at
# midnight every day.
#
# Collates all the logs (generated by various tools) into a common /var/s4logs
# directory structure. Logs are kept in GNU zip format to save space.
# When the original logs are in some non-text format, the appropriate utilities
# are used to generate text logs.
# This utility does NOT delete any of the "original" logs....
#
PATH=/sbin:/usr/sbin:/usr/bin:/usr/s4/etc:/usr/local/bin
export PATH
#
# Touch /var/adm/syslog.dated so that the standard cron find and remove
# does not eventually remove the top directory
#
touch /var/adm/syslog.dated
#
# Create directories if necessary
#
if [ ! -d /var/s4logs ]
then
   mkdir /var/s4logs
   chown root   /var/s4logs
   chgrp system /var/s4logs
   chmod 700    /var/s4logs
fi

#
# If an earlier run of this crashed, save the temporary directory
# before creating a new one
#
if [ -d /var/s4logs/tmpnew ]
then
   cd /var/s4logs
   INSTALL_TIME=`date +%Y-%m-%d-%H:%M:%S`
   mv tmpnew tmpnewsaved.$INSTALL_TIME
fi

mkdir /var/s4logs/tmpnew
chown root   /var/s4logs/tmpnew
chgrp system /var/s4logs/tmpnew
chmod 700    /var/s4logs/tmpnew

START_TIME=`now`
END_TIME=`now`

#
# Stop syslogd and binlogd before saving their data
#
/sbin/init.d/syslog stop

SYSLOG_HOME=`rcmgr get SYSLOG_HOME`
MORE_SYSLOG_DIRS=1
while [ $MORE_SYSLOG_DIRS = 1 ]
do
   NORM_SYSLOG_DIR=`ls -rt /var/adm/syslog.dated | head -1`
   NORM_SYSLOG_START_TIME=0
   S4_SYSLOG_START_TIME=0

   if [ "X$NORM_SYSLOG_DIR" != "X" ]
   then
      NORM_SYSLOG_START_TIME=`cvtsyslogdate $NORM_SYSLOG_DIR`
   fi

   S4_SYSLOG_DIR=`ls -rt $SYSLOG_HOME/var/adm/syslog.dated | head -1`

   if [ "X$S4_SYSLOG_DIR" != "X" ]
   then
      S4_SYSLOG_START_TIME=`cvtsyslogdate $S4_SYSLOG_DIR`
   fi

   if [ $NORM_SYSLOG_START_TIME = 0 ]
   then
      if [ $S4_SYSLOG_START_TIME = 0 ]
      then
         MORE_SYSLOG_DIRS=0
      else
         SYSLOG_DIR="$SYSLOG_HOME/var/adm/syslog.dated/$S4_SYSLOG_DIR"
         SYSLOG_START_TIME="$S4_SYSLOG_START_TIME"
      fi
   else
      if [ $S4_SYSLOG_START_TIME = 0 ]
      then
         SYSLOG_DIR="/var/adm/syslog.dated/$NORM_SYSLOG_DIR"
         SYSLOG_START_TIME="$NORM_SYSLOG_START_TIME"
      else
         if [ $S4_SYSLOG_START_TIME -lt $NORM_SYSLOG_START_TIME ]
         then
            SYSLOG_DIR="$SYSLOG_HOME/var/adm/syslog.dated/$S4_SYSLOG_DIR"
            SYSLOG_START_TIME="$S4_SYSLOG_START_TIME"
         else
            SYSLOG_DIR="/var/adm/syslog.dated/$NORM_SYSLOG_DIR"
            SYSLOG_START_TIME="$NORM_SYSLOG_START_TIME"
         fi
      fi
   fi

   if [ $MORE_SYSLOG_DIRS -ne 0 ]
   then
      if [ $SYSLOG_START_TIME -lt $START_TIME ]
      then
         START_TIME="$SYSLOG_START_TIME"
      fi

      cd $SYSLOG_DIR
      MORE_FILES=1
      while [ $MORE_FILES -ne 0 ]
      do
         FIRST_FILE=`ls -rt | head -1`

         if [ "X$FIRST_FILE" = "X" ]
         then
            MORE_FILES=0
         else
            TMP_TIME=`filemodtime $FIRST_FILE`
            if [ $END_TIME -lt $TMP_TIME ]
            then
               END_TIME="$TMP_TIME"
            fi
            cat $FIRST_FILE >> /var/s4logs/tmpnew/syslog.$FIRST_FILE
            rm $FIRST_FILE
         fi
      done

      cd ..
      rmdir $SYSLOG_DIR
   fi
   cd /var/s4logs/tmpnew
done

# We also need to take care of more binlogd/syslogd things here...

uerf > /var/s4logs/tmpnew/binlog.err.log
BINLOG_HOME=`rcmgr get BINLOG_HOME`
TMPTIME=`filemodtime $BINLOG_HOME/usr/adm/binary.errlog`
if [ $END_TIME -lt $TMPTIME ]
then
   END_TIME="$TMPTIME"
fi
echo -n "" > $BINLOG_HOME/usr/adm/binary.errlog

cd /var/s4logs/tmpnew
fgrep screend syslog.daemon.log > screend.all.log
fgrep REJECT screend.all.log > screend.reject.log
fgrep ACCEPT screend.all.log > screend.accept.log

# We are ready to start syslogs again
#
/sbin/init.d/syslog start &
sleep 3
#
# Store screend accumulated statistics
#
cd /var/s4logs/tmpnew
screenstat > screend.stat.log
/sbin/init.d/screen restart

#
# Store login usage statistics
#
cd /var/s4logs/tmpnew
last > login.last.log
ac -p > login.ac.log
who -a > login.cur.log

# Store password change logs
cat /var/adm/sialog > passwd-change.log
echo -n "" > /var/adm/sialog

TMPTIME=`filemodtime /var/adm/wtmp`
if [ $END_TIME -lt $TMPTIME ]
then
   END_TIME="$TMPTIME"
fi
TMPTIME=`filemodtime /var/adm/utmp`
if [ $END_TIME -lt $TMPTIME ]
then
   END_TIME="$TMPTIME"
fi
#
# Cannot erase wtmp and utmp easily. Causes problems elsewhere, since it breaks
# who -r until the next reboot.
#
# echo -n "" > /var/adm/wtmp
# echo -n "" > /var/adm/utmp
#
# Store audits in the same manner
#
/sbin/init.d/audit stop
sleep 1
cd /var/audit
cat auditd_cons > /var/s4logs/tmpnew/audit.cons.log
rm auditd_cons
TMPFILE=`ls | head -1`
while [ "X$TMPFILE" != "X" ]
do
   audit_tool $TMPFILE >> /var/s4logs/tmpnew/audit.log
   rm $TMPFILE
   TMPFILE=`ls /var/audit | head -1`
done
/sbin/init.d/audit start
#
# Store system crash data in the same manner
#
cd /var/adm/crash
NUMFILES=`ls | wc -l`
if [ $NUMFILES -ne 0 ]
then
   gzip *
   mv * /var/s4logs/tmpnew
fi

#
# Store Common format logs of the httpd web servers in the same manner
#
NUM_WEBSERVER_CONF=`rcmgr get NUM_WEBSERVER_CONF`
if [ "X$NUM_WEBSERVER_CONF" != "X" ]
then
   if [ $NUM_WEBSERVER_CONF -gt 0 ]
   then
      /sbin/init.d/webserver stop
      sleep 1

      # save logs for each web server

      CUR_WEBSERVER=0
      while [ $CUR_WEBSERVER -lt $NUM_WEBSERVER_CONF ]
      do
         WEBVAR_PREFIX="WEBSERVER_UNAME_"
         WEBVAR_IDX=`eval echo $WEBVAR_PREFIX$CUR_WEBSERVER`
         WEB_UNAME=`rcmgr get $WEBVAR_IDX`

         WEBVAR_PREFIX="WEBSERVER_HOMEDIR_"
         WEBVAR_IDX=`eval echo $WEBVAR_PREFIX$CUR_WEBSERVER`
         WEB_HOMEDIR=`rcmgr get $WEBVAR_IDX`

         cd /var/s4logs/tmpnew
         cat $WEB_HOMEDIR/logs/access_log  > $WEB_UNAME.access.log
         cat $WEB_HOMEDIR/logs/agent_log   > $WEB_UNAME.agent.log
         cat $WEB_HOMEDIR/logs/error_log   > $WEB_UNAME.error.log
         cat $WEB_HOMEDIR/logs/referer_log > $WEB_UNAME.referer.log

         cd $WEB_HOMEDIR/logs
         rm access_log agent_log error_log referer_log

         CUR_WEBSERVER=`expr $CUR_WEBSERVER + 1`
      done

      /sbin/init.d/webserver start
   fi
fi
#
# Compress logs in the tmpnew directory
#
cd /var/s4logs/tmpnew
gzip *.log
chmod 400 *
chown root *
chgrp system *
#
# We are finished! Rename the temporary directory to an appropriate
# directory.
#
cd /var/s4logs
cvtsecs $START_TIME > s4tmp1
read < s4tmp1 S4YEAR1 S4MONTH1 S4DAY1 S4HOUR1 S4MIN1 S4SEC

if [ $S4MONTH1 -lt 10 ]
then
   S4MONTH1="0$S4MONTH1"
fi
if [ $S4DAY1 -lt 10 ]
then
   S4DAY1="0$S4DAY1"
fi
if [ $S4HOUR1 -lt 10 ]
then
   S4HOUR1="0$S4HOUR1"
fi
if [ $S4MIN1 -lt 10 ]
then
   S4MIN1="0$S4MIN1"
fi

cvtsecs $END_TIME   > s4tmp1
read < s4tmp1 S4YEAR2 S4MONTH2 S4DAY2 S4HOUR2 S4MIN2 S4SEC

if [ $S4MONTH2 -lt 10 ]
then
   S4MONTH2="0$S4MONTH2"
fi
if [ $S4DAY2 -lt 10 ]
then
   S4DAY2="0$S4DAY2"
fi
if [ $S4HOUR2 -lt 10 ]
then
   S4HOUR2="0$S4HOUR2"
fi
if [ $S4MIN2 -lt 10 ]
then
   S4MIN2="0$S4MIN2"
fi

rm s4tmp1
NEWDIR="$S4YEAR1-$S4MONTH1-$S4DAY1-$S4HOUR1:$S4MIN1---$S4YEAR2-$S4MONTH2-$S4DAY2-$S4HOUR2:$S4MIN2"
mv tmpnew $NEWDIR

