/***************** unix_xd.c *********************/

/* This program is no longer used in the S4 environment.
   It is left for history's perusal (and endless wonder on how
   some people can waste their time)... */

/******************************************************************************
 * Copyright (c) 1995,1996 Donated to the public domain                       *
 *                                                                            *
 * Original author and maintainer: Marc Chatel   (mchatel@dial.oleane.com)    *
 * Last known maintainer:          Marc Chatel   (mchatel@dial.oleane.com)    *
 *                                                                            *
 * This file was created as part of the S4 (Secure System Setup Script) kit.  *
 * Permission is granted to any person or entity to do any of the following:  *
 *       a) use this file alone or in some other software                     *
 *       b) modify this file or include parts of this file in other files     *
 *       c) re-distribute this file AS IS or modified, for non-commercial     *
 *          or commercial purposes, alone or as part of some software package *
 *                                                                            *
 * No warranties of any kind, express or implied, on the functionality and    *
 * safety of the contents of this file. Use at your own risk!                 *
 *                                                                            *
 * If you do useful changes to this file (bug fixes, portability fixes,       *
 * enhancements), you should TRY to contact the current maintainer, who may   *
 * be maintaining a "latest greatest" version of the file. You do not HAVE    *
 * TO, but you should TRY. Promote software reuse! It helps everybody,        *
 * including you!                                                             *
 ******************************************************************************/

/* This is a small UNIX domain socket relay utility.
   Useful for running non-privileged syslog.
   I place this code in the public domain, free to everybody.

               Marc Chatel, 1995 */

/* This program is a VERY basic relay for datagrams for UNIX domain sockets.

   Takes in 5 parameters: parm1 parm2 parm3 parm4 parm5
   Binds to (listens for data from) UNIX domain socket passed as "parm1"
   Takes "parm2" as a username, "parm3" as a groupname, "parm4" as a directory.
   Does a chroot() to parm4, connects to UNIX domain socket passed as "parm4".
   Changes group id to the numeric value for parm3,
   Changes user id to the numeric value for parm2.
   Sits around and relays data from UNIX domain socket "parm1"
                               to socket "parm4".

   If there are error messages, they are written to file /tmp/unix_xd.log
   since syslog support is not working at the time this program runs. */

/* Daemon startup code taken almost verbatim from the
   "Unix Network Programming" book,
   Author W. Richard Stevens, Prentice Hall, ISBN 0-13-949876-1

   This is a skeleton of how a UNIX daemon process is supposed to be
   coded. This example is on pages 72-85 of the book.

   Typed in by Marc Chatel... */

   
#include   <standards.h>
#include   <sys/types.h>
#include   <sys/resource.h>
#include   <wait.h>
#include   <stdio.h>
#include   <sys/wait.h>
#include   <signal.h>
#include   <sys/param.h>
#include   <errno.h>
#include   <syslog.h>
#include   <sys/socket.h>
#include   <string.h>
#include   <sys/un.h>
#include   <unistd.h>
#include   <fcntl.h>
#include   <pwd.h>
#include   <grp.h>

extern int errno;

#ifdef SIGTSTP   /* True if BSD system */
#include  <sys/file.h>
#include  <sys/ioctl.h>
#endif

FILE *errfile = NULL;

void log_msg( my_msg )
char *my_msg;
{
   int     msglen  = 0;
   int     need_cr = 0;
   int     put_rc  = 0;
   char    datebuf[30];
   time_t  now;

   if (my_msg == NULL)    return;
   msglen = strlen( my_msg );
   if (msglen <= 0)       return;
   if (my_msg[msglen - 1] != '\n')  need_cr = 1;

   (void) time( &now );
   (void) sprintf( datebuf, "%.15s ", ctime( &now ) + 4 );
   put_rc = fputs( datebuf, errfile );
   if (put_rc == EOF)     return;
   put_rc = fputs( my_msg, errfile );
   if (put_rc == EOF)     return;
   if (need_cr)
   {
      datebuf[0] = '\n'; datebuf[1] = '\0';
      put_rc = fputs( datebuf, errfile );
   }
   (void) fflush( errfile );
}

int sig_child()
{
#ifdef BSD
   /* use the wait3() system call with the WNOHANG option */

   int         pid;
   /* union wait  status; */
   long   status;

   while ((pid=wait3(&status, WNOHANG, (struct rusage *) 0 )) > 0)
      ;
#endif
   return 0;
} /* sig_child() */

/* Detach a daemon process from login session context */

void daemon_start( ignsigcld )
int  ignsigcld;   /* nonzero -> handle SIGCLDs so zombies don't clog */
{
   register int   childpid, fd;

   /* If we were started by init (process 1) from the /etc/inittab file
      there is no need to detach. This test is unreliable due to an
      unavoidable ambiguity if the process is started by some other process
      and orphaned (i.e. if the parent process terminates before we are
      started) */

   if (getppid() == 1)  goto out;

   /* Ignore the terminal stop signals (BSD) */

#ifdef SIGTTOU
   signal( SIGTTOU, SIG_IGN );
#endif
#ifdef SIGTTIN
   signal( SIGTTIN, SIG_IGN );
#endif
#ifdef SIGTSTP
   signal( SIGTSTP, SIG_IGN );
#endif

   /* If we were not started in the background, fork and let the parent exit.
      This also guarantees the first child is not a process group leader. */

   if ((childpid = fork()) < 0)
   {
      log_msg( "can't form first child" );
      exit(1);
   }
   else
      if (childpid > 0) exit(0);  /* parent; */

   /* First child process. Disassociate from controlling terminal and
      process group. Ensure the process can't reacquire a new controlling
      terminal. */
#ifdef SIGTSTP   /* BSD */
   if (setpgrp( 0,getpid() ) == -1)
   {
      log_msg( "can't change process group" );
      exit(1);
   }

   if ((fd = open("/dev/tty",O_RDWR)) >= 0)
   {
      ioctl(fd, TIOCNOTTY, (char *)NULL); /* lose controlling tty */
      close(fd);
   }
#else   /* System V */
   if (setpgrp() == -1)
   {
      log_msg( "can't change process group" );
      exit(1);
   }

   signal(SIGHUP, SIG_IGN); /* immune from pgrp leader death */

   if ((childpid = fork()) < 0)
   {
      log_msg( "can't fork second child" );
      exit(1);
   }
   else
      if (childpid > 0)
         exit(0);   /* first child */

   /* second child */
#endif
   
   out:
   /* Close any open file descriptors */
   for (fd = 0; fd < NOFILE; fd++)   close(fd);

   errno = 0;  /* probably got set to EBADF from a close */

   /* Move the current directory to root, to make sure we are not
      on a mounted filesystem */

   chdir("/");

   /* Clear any inherited file mode creation mask. */

   umask(0);

   /* See if the caller isn't interested in the exit status of its
      children, and does not want to have the become zombies and clog up
      the system. With System V all we need to do is ignore the signal.
      With BSD, however, we have to catch each signal and execute the
      wait3() system call. */

   if (ignsigcld)
   {
#ifdef SIGTSTP
      int sig_child();

      signal( SIGCLD, sig_child );   /* BSD */
#else
      signal( SIGCLD, SIG_IGN );   /* System V */
#endif
   }
} /* daemon_start() */

int main( argc, argv )
int argc;
char **argv;
{
   uid_t          new_uid     = 0;
   uid_t          before_uid  = 0;
   gid_t          new_gid     = 0;
   struct passwd *pwdfile_ptr = NULL;
   struct group  *grpfile_ptr = NULL;

   errfile = fopen( "/tmp/unix_xd.log", "a" );
   if (errfile == NULL)   return 0;

   if (argc != 6)
   {
      log_msg(
       "I need 5 parms: source_socket username grpname directory dest_socket" );
      return 0;
   }

   before_uid = geteuid();
   if (before_uid != 0)
   {
      log_msg( "Need to be root in order to drop privileges" );
      return 0;
   }

   pwdfile_ptr = getpwnam( argv[2] );  /* Get entry for username */
   if (pwdfile_ptr != NULL)   new_uid = pwdfile_ptr->pw_uid;
   else
   {
      log_msg( "Cannot find username specified as second parameter" );
      return 0;
   }
   pwdfile_ptr = NULL;
   endpwent();

   grpfile_ptr = getgrnam( argv[3] ); /* Get entry for groupname */
   if (grpfile_ptr != NULL)   new_gid = grpfile_ptr->gr_gid;
   else
   {
      log_msg( "Cannot find groupname specified as third parameter" );
      return 0;
   }
   grpfile_ptr = NULL;
   endgrent();

   daemon_start(0);

   {
      int                 our_socket,      dest_socket;
      struct sockaddr_un  our_socket_addr, dest_socket_addr;
      int                 our_sock_fn_len, dest_sock_fn_len;
      int                 our_rc;

      our_sock_fn_len = strlen( argv[1] );
      if (our_sock_fn_len > 100)
      {
         log_msg( "OUR socket filename is too long" );
         return 0;
      }
      our_socket_addr.sun_family = AF_UNIX;
      strcpy( our_socket_addr.sun_path, argv[1] );
#ifdef _SOCKADDR_LEN
      our_socket_addr.sun_len = our_sock_fn_len + 1;
#endif

      our_socket = socket( AF_UNIX, SOCK_DGRAM, 0 );
      if (our_socket < 0)
      {
         log_msg( "Cannot get OUR socket" );
         return 0;
      }

      our_rc = bind( our_socket, (struct sockaddr *) &our_socket_addr,
                      sizeof( struct sockaddr_un ));

      if (our_rc != 0)
      {
         log_msg( "BIND to OUR socket failed" );
         return 0;
      }

      our_rc = connect( our_socket, (struct sockaddr *) &our_socket_addr,
                        sizeof( struct sockaddr_un ));

      if (our_rc != 0)
      {
         log_msg( "CONNECT to OUR socket failed" );
         return 0;
      }

      if (initgroups( argv[2], new_gid ))
      {
         log_msg( "cannot do initgroups" );
         return 0;
      }
      endgrent();

      if (chroot( argv[4] ) != 0)
      {
         log_msg( "cannot chroot to directory specified in fourth parm");
         return 0;
      }
      if (chdir("/") != 0)
      {
         log_msg( "cannot chdir to directory specified in fourth parm");
         return 0;
      }

      dest_sock_fn_len = strlen( argv[5] );
      if (dest_sock_fn_len > 100)
      {
         log_msg( "DEST socket filename is too long" );
         return 0;
      }
      dest_socket_addr.sun_family = AF_UNIX;
      strcpy( dest_socket_addr.sun_path, argv[5] );
#ifdef _SOCKADDR_LEN
      dest_socket_addr.sun_len = dest_sock_fn_len + 1;
#endif

      dest_socket = socket( AF_UNIX, SOCK_DGRAM, 0 );
      if (dest_socket < 0)
      {
         log_msg( "Cannot get DEST socket" );
         return 0;
      }

      our_rc = connect( dest_socket, (struct sockaddr *) &dest_socket_addr,
                        sizeof( struct sockaddr_un ));

      if (our_rc != 0)
      {
         log_msg( "CONNECT to DEST socket failed" );
         return 0;
      }

      /* Now let us drop privilege */

      if (setgid( new_gid ) != 0)
      {
         log_msg( "Failed to change real group id" ); return 0;
      }

      if (setegid( new_gid ) != 0)
      {
         log_msg( "Failed to change effective group id" ); return 0;
      }

      if (setuid( new_uid ) != 0)
      {
         log_msg( "Failed to change real user id" ); return 0;
      }

      if (seteuid( new_uid ) != 0)
      {
         log_msg( "Failed to change effective user id" ); return 0;
      }

      log_msg( "Unix domain relay starting" );

      while (1)
      {
         char  bigbuf[65536];
         int   in_bytes, sent_out;

         in_bytes = recv( our_socket, bigbuf, 60000, 0 );
                
         if (in_bytes > 0)
         {
            sent_out = send( dest_socket, bigbuf, in_bytes, 0 );
            if (sent_out != in_bytes)
            { 
               log_msg( "Failed to send all the data" );
               return 0;
            }
         }
      }
   }
}
