/********************** popdeliver.c **********************/

/******************************************************************************
 * Copyright (c) 1995,1996 Donated to the public domain                       *
 *                                                                            *
 * Original author and maintainer: Marc Chatel   (mchatel@dial.oleane.com)    *
 * Last known maintainer:          Marc Chatel   (mchatel@dial.oleane.com)    *
 *                                                                            *
 * This file was created as part of the S4 (Secure System Setup Script) kit.  *
 * Permission is granted to any person or entity to do any of the following:  *
 *       a) use this file alone or in some other software                     *
 *       b) modify this file or include parts of this file in other files     *
 *       c) re-distribute this file AS IS or modified, for non-commercial     *
 *          or commercial purposes, alone or as part of some software package *
 *                                                                            *
 * No warranties of any kind, express or implied, on the functionality and    *
 * safety of the contents of this file. Use at your own risk!                 *
 *                                                                            *
 * If you do useful changes to this file (bug fixes, portability fixes,       *
 * enhancements), you should TRY to contact the current maintainer, who may   *
 * be maintaining a "latest greatest" version of the file. You do not HAVE    *
 * TO, but you should TRY. Promote software reuse! It helps everybody,        *
 * including you!                                                             *
 ******************************************************************************/

/* popdeliver
   ----------
   This is a "glue" program. It is only useful in very precise conditions...

   The following assumptions are made:

   a) sendmail is running non-privileged and chrooted, but you still need
         to handle mail delivery to a local POP area.

   b) We want to separate sendmail processing (the mail queue, sendmail config,
      and so on) from the "POP" mailboxes. In other words, we would like to
      make it difficult for sendmail to look into the POP mailboxes, and we
      would like to make it difficult for the POP mail delivery program to play
      with the sendmail config. To achieve this, we will run "POP delivery"
      in a chroot area UNDER the chroot area where sendmail itself is running.
      To do this, this program (popdeliver) must be setuid root in order to
      do the 2nd chroot successfully...

   c) popdeliver should be statically linked. No shared libraries please,
      as this may introduce extra vulnerabilities in the chroot area.

   d) Many dangerous parameters are pre-set in this source code.

       - the directory of the chroot area UNDER the sendmail chroot area
         is preset to "/pop_delivery"
       - the name of the final mail delivery program is preset to
         /bin/spop (from the point of view of the pop_delivery chroot area)

   localdeliver performs the following actions in order:
   -----------------------------------------------------
   0. STARTS WITH ROOT PRIVILEGE, SINCE IT IS INVOKED SETUID
   1. calculates the UNIX group id for the group name under which the POP
      server operates (preset in this source by the S4 install script)
   2. calculates the UNIX user id for the user name under which the POP
      delivery system operates (preset in this source by the S4 install script)
   3. does a chroot to "/pop_delivery", then drops privilege to the
      above-mentioned userid and groupid
   4. Rebuilts the original command-line parameters, and executes
      /bin/spop with those parameters
*/

#include <syslog.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "drop_priv.h"

#ifdef USE_SYSLOG
#ifdef USE_PRTF
   I need either USE_SYSLOG or USE_PRTF defined, not both
#else
#endif
#else
#ifdef USE_PRTF
#else
   I need either USE_SYSLOG or USE_PRTF defined
#endif
#endif

char *called_prog_name = "/bin/spop";

int main( argc, argv )
int argc;
char **argv;
{
   if (!drop_priv( "pop", "S4POPGNAME", "/pop_delivery" ))
   {
#ifdef USE_SYSLOG
      syslog( LOG_ERR, "Cannot drop privileges\n" );
#else
      fprintf( stderr, "Cannot drop privileges\n" );
#endif
      return 1;
   }

#ifdef 0
#ifdef USE_SYSLOG
      syslog( LOG_INFO, "After drop_priv" );
#else
      fprintf( stderr, "After drop_priv\n" );
#endif
#endif

   {
      char **arg_buf = NULL;
      int    our_idx = 0;

      arg_buf = (char **) malloc( sizeof(char *) * (argc + 1) );

      if (arg_buf == NULL)
      {
#ifdef USE_SYSLOG
         syslog( LOG_ERR, "Cannot allocate parameter area\n" );
#else
         fprintf( stderr, "Cannot allocate parameter area\n" );
#endif
         return 1;
      }

      arg_buf[ 0 ] = called_prog_name;

      for (our_idx = 1; our_idx < argc; our_idx++)
         arg_buf[ our_idx ] = argv[ our_idx ];

      arg_buf[ argc ] = NULL;

      (void) execv( called_prog_name, arg_buf );
   }
   return 1;
}
